import pandas as pd
import numpy as np
from scipy import interpolate
from scipy.interpolate import interp1d
import warnings
import copy
import math
from scipy import signal
from scipy.optimize import curve_fit
import xarray as xr
from sympy import *
import sympy

from scipy import stats
from sympy import *
import sympy

#-----------------------------------------
# operators to be applied on xr-data, or pandas-timeseries

# def understand_xr(ds):
    

    

# def detrend(ds,deseason=True)




# def corr
# def add
# def sub 
#
#
#
#
#

def sl_pd_series(ydata,xdata):
    """
    xdata - time-vector
    ydata - data-vecttor
    """
    
    xdata=pd.to_datetime(xdata)
    series=pd.Series(ydata, index=xdata)    
    series = series[pd.notnull(series)]    
    return series

def sl_trend_prepare(ydata,xdata,**kwargs):
    series=sl_pd_series(ydata,xdata)
    series=series[~np.isnan(series)].dropna()
  
    if len(series.dropna())<10:
        out=[np.nan]*10
    else:
        out=t_fit_annual_cycle(series,**kwargs)
        #[popta[1],Uncer_minann2[1],AMv,UncAm,y_minann_and_trend,
                #pd.Series(y_mintrend, index =series.index)]#,pd.Series(y_minann_and_trend, index =series.index)]
    
    
    return np.array(out[0:8])



def trend_sub(self,**kwargs):      
    out=xr.apply_ufunc(sl_trend_prepare, self.data,self.data.time,
                      input_core_dims=[["time"],["time"]],output_core_dims=[['var']],
                             kwargs=kwargs,dask = 'allowed', vectorize = True)

    names=['trend','trend_un','AC','AC_unc','offset','acos','asin','phase']
    long_names=['linear trend','trend uncertainties','annual cycle','annual cylce uncertainties',
                'offset','acos','asin','annual cycle phase phase']
    units=['m/year','m/year','m','m','m','m','m','rad']
    i=0
    ds_array=[]
    if self.typ=='xra':
        print('auto-select first variable ',self.var)
        out=out[self.var]
    
    
    if len(self.shape)>2:
        for name in names:
            ds_array.append(out[:,:,i].rename(name).astype(float))
            ds_array[i].attrs={'standard_name': [names[i]], 'long_name': long_names[i],'_FillValue' : 'nan', 'units': units[i]}
            i=i+1
        out1=xr.merge(ds_array,compat='override')
    else:

        for name in names:
            ds_array.append(out[:,i].rename(name).astype(float))
            ds_array[i].attrs={'standard_name': [names[i]], 'long_name': long_names[i],'_FillValue' : 'nan', 'units': units[i]}
            i=i+1
        out1=xr.merge(ds_array,compat='override') 
        
    out1=out1.assign_coords({"time": self.flat_y.mean()})
    out1=out1.expand_dims('time')      
    return out1





#-------------------------------------------
# main fit



def t_fit_annual_cycle(series,C=0.001,P=1.5,**kwargs):
    """
    fit annual cycle and trend 
    use prais-winston method
    
    Parameters:
    ------------
    series
    
    autocorrelation
    
    Returns:
    ------------
    
    popt : array
        Optimal values for the parameters so that the sum of the squared residuals of 
        f(xdata, *popt) - ydata is minimized
        
        popt[0:3] : offset, trend, ...
    
    """
    de_season=kwargs['de_season']
    semi=kwargs['semi']
    cubic=kwargs['cubic']    
    de_season_without_trend=kwargs['de_season_without_trend']
    trend_only=kwargs['trend_only']
        
    
    series=series[~np.isnan(series)]
    year=pd.Timedelta('365.2422 days')
    f=year.total_seconds()   
    xdata=(series.index-pd.to_datetime('1990-01-01')).total_seconds().values/f
    ydata=series.values
    
    
    
    if semi:
        popt, pcov = curve_fit(annual_cycle_semi, xdata, ydata)   
        y_minann=ydata-annual_cycle_semi(xdata,0,0,popt[2],popt[3],popt[4],popt[5])
        DoF=len(xdata)-6 #Degrees of Freedom N-(A,B,C,D)
    elif trend_only:
        popt=[0]*10
        pcov=[0]*10
        poptb, pcovb = curve_fit(ftrend_only, xdata, ydata)  
        popt[0:2]=poptb[0:2]
        pcov[0:2]=pcov[0:2]
        y_minann=ydata  
        DoF=len(xdata)-2 #Degrees of Freedom N-(A,B,C,D)
    else:    
        popt, pcov = curve_fit(annual_cycle, xdata, ydata)
        y_minann=ydata-annual_cycle(xdata,0,0,popt[2],popt[3])
        DoF=len(xdata)-4 #Degrees of Freedom N-(A,B,C,D)

        
    t_critical=stats.t.ppf(1-0.025,DoF) #95 % confidence interval    
    # option to start preis winston


      
        
    SE_noCO=np.sqrt(np.diag(pcov))
    Uncer_noCO=SE_noCO*t_critical 
    # subtract annual cycle

    if de_season:
        y_mintrend =ydata-annual_cycle(xdata,popt[0],popt[1],popt[2],popt[3])    

    else:
        y_mintrend =ydata-annual_cycle(xdata,popt[0],popt[1],0,0)

    if de_season_without_trend:
        y_mintrend =ydata-annual_cycle(xdata,0,0,popt[2],popt[3])    



    #plt.plot(xdata,annual_cycle(xdata,0,popt[1],0,0))
    #plt.plot(xdata,y_mintrend)

    # now estimate trend
    if cubic:
        #print len(y_minann)
        popta, pcova = curve_fit(trend_only_and_cubic, xdata, y_minann,bounds=([-np.inf,-np.inf,-np.inf,1.,-np.inf], [np.inf,np.inf,np.inf,25.,np.inf]),max_nfev=100000)       
        #popta, pcova = curve_fit(annual_cycle_semi, xdata, ydata)
        #a + b*x + c*np.cos((2*np.pi*x/d)+e)
        y_mintrend =ydata-trend_only_and_cubic(xdata,0,0,popta[2],popta[3],popta[4]) 
        #print popta
    else:        
        if trend_only:
            popta=popt
            pcova=pcovb
       
        else:
            popta, pcova = curve_fit(ftrend_only, xdata, y_minann) #popta offset + trend

    y_minann_and_trend=ydata-annual_cycle(xdata,0,popta[1],popt[2],popt[3])

    SE_a=np.sqrt(np.diag(pcova))    
    DoF1=len(xdata)-2 #two options
    Uncer_minann2=SE_a*stats.t.ppf(1-0.025,DoF1)    
    # error propagation for phase + amp
    c=Symbol('c')
    d=Symbol('d')       
    amp=sympy.sqrt(c**2+d**2)
    ph=sympy.atan(c/d)
    
    if trend_only:
        UncAm=0
        Uncph=0
        AMv=0
        PHv=0
    else:
        UncAm=error_prop(amp,[c,d],np.array([popt[2],popt[3]]),np.array([Uncer_noCO[2],Uncer_noCO[3]]))
        Uncph=error_prop(ph,[c,d],np.array([popt[2],popt[3]]),np.array([Uncer_noCO[2],Uncer_noCO[3]]))

        AMv=AMP(popt[2],popt[3])
        PHv=PH(popt[2],popt[3])        

    if cubic:
        UncAm=popta[3]
        AMv=popta[2]
    ##print popt


    return [popt[1],Uncer_minann2[1],AMv,UncAm,popt[0],popt[2],popt[3],PHv,y_minann_and_trend,
            pd.Series(y_mintrend, index =series.index)]#,pd.Series(y_minann_and_trend, index =series.index)]

        
        


        

def AMP(C,D):
    return np.sqrt(C**2+D**2)

def PH(C,D):
    if C>0:
        return np.arctan(D/C)+np.pi
    else: 
        return np.arctan(D/C)

def error_prop(f,varlist,vals,errs):
    n=len(varlist)
    df_at_x=[]
    for i in range(n):
        sig = f.diff(varlist[i])
        t=sig
        for j in range(n):
            t=t.subs({varlist[j]:vals[j]})
        df_at_x.append(t)
    error1 = sympy.sqrt(np.sum((np.asarray(df_at_x)**2)*(errs**2)))
    
    return error1

# fit functions

def ftrend_only(x,a,b):
    return  a + b*x

def trend_only_and_cubic(x,a,b,c,d,e): #,c,d):
    return  a + b*x + c*np.cos((2*np.pi*x*d)+e) #+c*(x**2) #+d*(x**3)

def trend_only_vec(x,a,b):
    return  a*x[:,3] + b*x[:,0]

def two_gauss( x, c1, mu1, sigma1, c2, mu2, sigma2):
    res =   c1 * np.exp( - (x - mu1)**2.0 / (2.0 * sigma1**2.0) ) \
          + c2 * np.exp( - (x - mu2)**2.0 / (2.0 * sigma2**2.0) )
    return res

def annual_cycle(x,a,b,c,d):
#    year=pd.Timedelta('365.2422 days')
#    f=year.total_seconds()    
    return  a + b*x + c*np.cos(2*np.pi*x) + d*np.sin(2*np.pi*x)

def annual_cycle_semi(x,a,b,c,d,e,f):
#    year=pd.Timedelta('365.2422 days')
#    f=year.total_seconds()    
    return  a + b*x + c*np.cos(2*np.pi*x) + d*np.sin(2*np.pi*x) + e*np.cos(4*np.pi*x) + f*np.sin(4*np.pi*x)

def f_linear(x,a):
    return a*x   


def annual_cycle_vec(x,a,b,c,d):
#    year=pd.Timedelta('365.2422 days')
#    f=year.total_seconds()    
    return  a*x[:,3] + b*x[:,0] + c*x[:,1] + d*x[:,2]









#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Thu May  9 16:15:59 2019

@author: oelsmann
"""


import numpy as np
import pandas as pd
from scipy.stats import norm
from scipy.optimize import curve_fit

    
class slt_object:
    """
    sea_level_tool Object
    
    - accepts xr, pd or float objects
    - extracts necessary information for statistical tools to be applied (sl)
    - class sl needs slt_object
    
    information on data-set
    helps to understand and pack the data to be used by sl()
        typ = check_type(self.data)
        data = data        
        shape =
        flat_x =
        flat_y =
        flat_numeric_y =
        operator =
    
    """

    
    def __init__(self,data):   
        

        
        def understand_data(self):
            """
            understand data-type (if pandas timeseries or xr)
            and dimension
            """    

        self.data = data    
        
    
    @property
    def typ(self):
        """
        xr, pandas, float
        """

        str_typ = str(type(self.data))
        if str_typ=="<class 'pandas.core.frame.DataFrame'>":
            typ='pd'
        if str_typ=="<class 'pandas.core.series.Series'>":
            typ='pd'                
        elif str_typ=="<class 'float'>":
            typ='float'
        elif str_typ=='xarray.core.dataset.Dataset':
            typ='xra'
        elif str_typ== "<class 'xarray.core.dataarray.DataArray'>":
            typ='xr'
        return typ     
    
    @property    
    def var(self):
        if self.typ == 'xra':
            return list(self.data.keys())[0]
        elif self.typ == 'xr' or self.typ == 'pd':
            return self.data.name
        else:
            return ''
    
    
    @property
    def shape(self):
        if self.typ == 'xra':
            return self.data[self.var].values.shape            
        elif self.typ == 'xr':
            return self.data.values.shape
        elif self.typ == 'pd':
            return self.data.shape
        else:
            return ()        
        
        #return get_shape(self)   
    @property
    def flat_x(self):
        if self.typ == 'xra':

            if len(self.shape) > 2:
                 flat_x= self.data[self.var].values.reshape(self.shape[0],self.shape[1]*self.shape[2])
            else:           
                 flat_x= self.data[self.var].values
        elif self.typ == 'xr':

            if len(self.shape) > 2:
                 flat_x= self.data.values.reshape(self.shape[0],self.shape[1]*self.shape[2])
            else:           
                 flat_x= self.data.values                

        elif self.typ == 'pd':
             flat_x=self.data.index
        else:
             flat_x= self.data
        return flat_x        
        #return self.__flat_x
    @property
    def flat_y(self):
        if self.typ == 'xr' or self.typ == 'xra':
            if 'time' in self.data.dims:
                timeser = self.data.time
            else: 
                timeser=np.array([0])

        elif self.typ == 'pd':
            timeser =self.data.index
        else:
            timeser = np.array([0])
        if len(timeser) > 1:                
            timeser=pd.to_datetime(timeser.values)    
        return timeser
        #return self.__flat_y
    @property
    def flat_y_numeric(self):
        year=pd.Timedelta('365.2422 days')
        f=year.total_seconds()   
        if len(self.flat_y)>1:
            return (self.flat_y-pd.to_datetime('1990-01-01')).total_seconds().values/f
        else:
            return (pd.to_datetime('2000-01-01')-pd.to_datetime('1990-01-01')).total_seconds()/f                

        #return self.__flat_y_numeric     

            

class sl(slt_object):
    """
    statistical tools
    inherits attributes from sl_obj
    
    data = data-obj -> slt_object.data
    data2= 2nd obj tto be added, subtracted or correlated with
    """  
    
  
    
    def plot(self,**kwargs):   
        
        if not 'var' in kwargs:
            kwargs['var']=[self.var]
            if self.typ=='xr':
                kwargs['var']=['no_var']                
        plt_xr_map(self.data,**kwargs)
        
        
    def norm_data(self,data2):
        """
        
        """
        if data2.typ=='float':
            return data2
        
        elif data2.typ=='pd':
            data2.data=self.data*0+data2*(self.data*0+1).values[0,:,:,np.newaxis]
        
        else:
            data2.data=self.data*0+data2.data
        
        return data2
        
        
        

    def add(self,data2):
        data2_norm=norm_data(self,sl(data2))
        #self.data=xr.apply_ufunc(sl_trend_prepare, self.data,self.data.time,
        #                  input_core_dims=[["time"],["time"]],output_core_dims=[['var']],
        #                         kwargs=kwargs,dask = 'allowed', vectorize = True)     
        
        self.data=self.data+data2_norm.data
        
    def cor(self,data2):
        
        print(cor)
    
    def sub(self,data2):
        print(cor)    

    
    def detrend(self,**kwargs):
        """
        subtract trend and annual cycle
        
        **kwargs ={'de_season' : 'False','semi' : 'False'}
        """
        #kwargs.setdefault('detrend', True)        
        kwargs['dtrend']=True
        out=trend_sub(self,**kwargs) # function in sl_stats
    
        

        if len(self.shape)>2:
            detrend_arr=annual_cycle(self.flat_y_numeric,out['offset'].values[0,:,:,np.newaxis],
                     out['trend'].values[0,:,:,np.newaxis],
                     out['acos'].values[0,:,:,np.newaxis],
                     out['asin'].values[0,:,:,np.newaxis])        
            self.data=self.data-np.swapaxes(np.swapaxes(detrend_arr,0,2),1,2)
            
        else:
            detrend_arr=annual_cycle(self.flat_y_numeric,out['offset'].values[0,:,np.newaxis],
                     out['trend'].values[0,:,np.newaxis],
                     out['acos'].values[0,:,np.newaxis],
                     out['asin'].values[0,:,np.newaxis])             
            self.data=self.data-np.swapaxes(detrend_arr,0,1)
                              
        self.data.attrs={'standard_name': self.var, 'long_name': 'detrended '+self.var,'_FillValue' : 'nan'}                   
        return self  
    

    
    def trend(self,de_season=False,cubic=False,semi=False,
                       de_season_without_trend=False,trend_only=False):
        
        kwargs={'de_season':de_season,'cubic':cubic,'semi':semi,
                       'de_season_without_trend':de_season_without_trend,'trend_only':trend_only}
        
        self.data=trend_sub(self,**kwargs)  
        return self
    
    
    def selmon(self,months):
        
        months=np.asarray(months)
        out=self.data
        if self.typ == 'xr' or self.typ == 'xra':

            self.data=out[np.isin(pd.to_datetime(out.time.values).month,months),:,:]

        elif self.typ == 'pd':
            self.data=out[np.isin(pd.to_datetime(out.time.values).month,months)].dropna()
        else:
            '! no pandas or xarray data-type !'        
        return self         
        

    
    def yearmean(self,):
        if self.typ == 'xr' or self.typ == 'xra':
            self.data=self.data.resample(time="Y").mean()
        elif self.typ == 'pd':
            self.data=self.data.resample('Y').mean()
        else:
            '! no pandas or xarray data-type !'
        return self
 
    def monmean(self,):
        if self.typ == 'xr' or self.typ == 'xra':
            self.data=self.data.resample(time="M").mean()
        elif self.typ == 'pd':
            self.data=self.data.resample('M').mean()
        else:
            '! no pandas or xarray data-type !'
        return self       
  
    def seasmean(self,seas='DJF'):
        ll=['DJF','MAM','JJA','SON']
        ll2=[12,3,6,9]
        ll3=[3,6,9,12]
        
        if self.typ == 'xr' or self.typ == 'xra':
            month=ll3[ll.index(seas)]
            out=self.data.resample(time="QS-DEC").mean()
            if self.typ == 'xra': 
                out=out[self.var].shift(time=1)
            else:
                out=out.shift(time=1)
                
            time_new=out[pd.to_datetime(out.time.values).month==month,:,:]
            self.data=time_new.resample(time="Y").mean()
            
            
            
            
        elif self.typ == 'pd':
            month=ll2[ll==seas]
            out=self.data.resample("QS-DEC").mean()
            self.data=out.where(out.index.month==month).dropna().shift(2, freq ='MS').resample('Y').mean()

        else:
            '! no pandas or xarray data-type !'        
        return self   
    def timavg(self):
        if self.typ == 'xr' or self.typ == 'xra':
            self.data=self.data.mean(dim=['time'])
        elif self.typ == 'pd':
            self.data=self.data.mean(axis=0)
        else:
            '! no pandas or xarray data-type !'
        return self
    
 

    
class sat:
    """
    object contains time series and information for altimetry measurement
    
    """
    
    def __init__(self,mission,data,loc,dist_coast,frequency,correction_version,box_length):
        self.mission = mission
        self.loc = loc 
        self.data = data
        self.dist_coast =dist_coast
        self.frequency =frequency
        self.correction_version =correction_version
        self.box_length =box_length
        
        
        
class tidegauge:
    """
    object contains time series and information for tide gauge measurement
    
    """
    def __init__(self,name,data,loc,info,months,flag,tgindex):
        self.name = name
   
        if (flag == 0) or (flag == 2) or (flag == 3): # flag = 1: valid
            self.data = data
        else:
            self.data = np.nan              
        self.loc = loc #lat lon
        self.info = info
        self.months = months
        self.tgindex = tgindex
    def correct_xtr(self):  #exclude 999%percentiles
        self.data=gauss_percentiles(self.data)

        
        
def gauss_percentiles(data):
    
    gauss_fit = norm.fit(data.sealevel[~np.isnan(data.sealevel)])
    quant_gauss_up=norm.ppf(0.9998,gauss_fit[0],gauss_fit[1])
    quant_gauss_down=norm.ppf(0.0002,gauss_fit[0],gauss_fit[1])
    correct=data.mask(data.sealevel>quant_gauss_up)
    correct=correct.mask(correct.sealevel<quant_gauss_down)    
    return correct        



import pandas as pd
import numpy as np
#from scipy import interpolate
#from scipy.interpolate import interp1d
import seaborn as sns
#from mpl_toolkits.basemap import Basemap
import matplotlib.pyplot as plt
#from matplotlib.pyplot import figure
#from statsmodels.graphics.tsaplots import plot_acf
#from matplotlib.lines import Line2D
import xarray as xr
#import matplotlib.collections as collections
import cartopy.crs as ccrs
import cartopy.feature as cfeature
from cartopy.mpl.gridliner import LONGITUDE_FORMATTER, LATITUDE_FORMATTER
import matplotlib.cm as mpl_cm
import iris
import iris.quickplot as qplt
import os
    
def plt_make_ticks(lat):
    
    latrange=np.abs(np.max(lat)-np.min(lat))
    
    res=int((latrange*0.4))
    
    xticks=np.arange(-180,360,res)
    yticks=np.arange(-90,90,res)
    
    return xticks,yticks
    
def get_divider(num):
    dividers=[]
    if num > 1:
       # check for factors
        for i in range(2,num):
            if (num % i) == 0:
                dividers.append(i)
    else:
        dividers.append(1)

    return np.array(dividers)

def color_range(data,range_in,option='std'):
    """
    calc colorrange
    """
    if range_in[1]==0:
        if option == 'std':
            std=np.nanquantile(data, .25)        
            med=np.nanmedian(data)
            std=med-std
            minv=med-3*std
            maxv=med+3*std

        elif option =='minmax':
            minv=np.nanmin(data)
            maxv=np.nanmax(data)
    else:
        minv=range_in[0]
        maxv=range_in[1]   
    return minv,maxv
    
    

def fit_2D_dimension(num):
    
    dividers=get_divider(num)
    if len(dividers)>0:
        x=dividers[np.argmin(np.abs((dividers-np.sqrt(num))))]
        y=int(num/x)
    else:
        num=num+1
        x,y,num = fit_2D_dimension(num)
    return x,y,num

def gl_props(gl,scaler):
    gl.xlabel_style = {'size': 80*scaler}
    gl.ylabel_style = {'size': 80*scaler}
    gl.xformatter = LONGITUDE_FORMATTER
    gl.yformatter = LATITUDE_FORMATTER
    gl.ylabels_left = True
    gl.ylabels_right = False    
    return gl

def plt_xr_map_ax(ax,fig,data,lat,lon,land_50m,title,year,range_in,unit,
                  function='scatter',msize=0.3,color_map=''):
    """
    ax-object for xr_map plot
    """
    xticks,yticks=plt_make_ticks(lat)
    
    ax.add_feature(land_50m)
    ax.coastlines('10m')
    ax.set_aspect(aspect=1.6)
    minv,maxv = color_range(data,range_in,option='std')
    if function=='scatter':
        im = ax.scatter(lon,lat,c=data,s=msize,vmin=minv,vmax=maxv,cmap=color_map,transform=ccrs.Geodetic())
    elif function =='contour':
        ax.contourf(lon,lat,data,transform=ccrs.PlateCarree())        

    gl=ax.gridlines(draw_labels=True,xlocs=xticks, ylocs=yticks, alpha=0.25)
    gl=gl_props(gl,0.15)
    #ax.set_title(title)
    clb = fig.colorbar(im, ax=ax, orientation='vertical',shrink=0.7)
    clb.ax.set_title(unit)    
    plt.text(0.5, 1.12, title,
         horizontalalignment='center',
         fontsize=15,
         transform = ax.transAxes)
    plt.text(0.8, 0.15, year,
         horizontalalignment='center',
         fontsize=15,
         transform = ax.transAxes)    
    #cbar=fig.colorbar(im,cax=ax, orientation='vertical',shrink=0.5)   
    #cbar.ax.set_xlabel('RMS [mm/year]')    

    
    return ax

def make_map_data(ds,vari,date_ini,function):
    print(date_ini)
    
    if vari=='no_var':
        ds_f=ds
    else:
        ds_f=ds[vari] 
    if 'time' in ds.dims:
  
    
        if function=='scatter':
            lat=ds_f.loc[date_ini].lat.values.flatten()
            lon=ds_f.loc[date_ini].lon.values.flatten()
            data=ds_f.loc[date_ini].values.flatten()            
            #lat=ds_f.where(ds_f.time==date_ini,drop=True).lat.values.flatten()
            #lon=ds_f.where(ds_f.time==date_ini,drop=True).lon.values.flatten()
            #data=ds_f.where(ds_f.time==date_ini,drop=True).values.flatten()
        elif function =='contour':
            lat=ds_f.where(ds_f.time==date_ini,drop=True).lat.values
            lon=ds_f.where(ds_f.time==date_ini,drop=True).lon.values
            data=ds_f.where(ds_f.time==date_ini,drop=True).values  
    else:
        if function=='scatter':
            print(vari)
            lat=ds_f.lat.values.flatten()
            lon=ds_f.lon.values.flatten()
            data=ds_f.values.flatten()        
   
    return lat,lon,data,ds_f

def adjust_array_len(arrayin,plot_number):
    
    len_ar=len(arrayin)
    if len_ar < plot_number and len_ar > 1:
        arrayin = arrayin*int((plot_number/len_ar+1))
    elif len_ar < plot_number and len_ar == 1:
        arrayin = arrayin*plot_number
    return arrayin
    
    

def arrays_options(ds,var,time_in,ranges,units):
    
    if time_in[0]==0 and 'time' in ds.dims:
        time_in=[ds.time[0].values]
    
    plot_number=len(var)*len(time_in)
    ranges = adjust_array_len(ranges,plot_number)
    units = adjust_array_len(units,plot_number)  
    
    if len(var) == plot_number:  
        time_in = adjust_array_len(time_in,plot_number)          
    elif len(time_in) == plot_number:
        var = adjust_array_len(var,plot_number)           
    else:
        var = adjust_array_len(var,plot_number)   
        time_in = adjust_array_len(time_in,plot_number)    
        
        
        
#    if time_in[0]==0:
#        time_in=[ds.time[0].values]#

#    if len(time_in)>1:
#        date_in=time_in
#        var=var * len(time_in)
#        plot_number=len(time_in)
#    elif len(var)>1:
#        date_in=time_in * len(var)
#        plot_number=len(var)
#        if time_in[0]==0:
#            date_in=[]
#            for i in range(plot_number):
#                date_in.append(ds.var[i-1].time[0].values)            
#    else:
#        print('auto-print')
#        date_in=time_in * 1
#        var=var * 1
#        plot_number=1    
#    if (plot_number > 1) and (len(ranges)<2):
#        ranges=[[0,0]]*plot_number
#    if plot_number is not len(units):
#        units=units*plot_number  
    return plot_number,var,time_in,ranges,units

def plt_xr_map(ds,var=['ssh'],time=[0],ranges=[[0,0]],units=[''],
               domain='auto',save='False',save_dir='',save_name='',msize=0.3):
    """
    plot map of xr at timestep
    
    var: list of variables in data
    time: list of time-steps to plot
    ranges: list of [vmin,vmax] values of plotted data-range
    units: list of units
    domain: lon, lat domain [lonmin,lonmax,latmin,latmax]
    save: bool (save figure)
    save_dir: save-directory
    
    
    """

    cmap=mpl_cm.get_cmap('brewer_RdBu_11')
    proj=ccrs.PlateCarree()
    land_50m = cfeature.NaturalEarthFeature('physical', 'land', '10m',
                                    edgecolor='face',
                                    facecolor='#f7f7f7')#cfeature.COLORS['land'])

    # to be automised (loop over axis)
    print(var)
    plot_number,var,date_in,ranges,units=arrays_options(ds,var,time,ranges,units)

    function='scatter'
    xp,yp,nump=fit_2D_dimension(plot_number)
    size=7
    fig =plt.figure(figsize=(xp*size*1.1, yp*size))
    
    
    for i in range(1,plot_number+1):
        date_ini=pd.Timestamp(date_in[i-1])
        vari=var[i-1]
        lat,lon,data,ds_f=make_map_data(ds,vari,date_ini,function)
        print('date: ',date_ini)
        if 'long_name' in ds_f.attrs:
            title=ds_f.long_name        
        else:
            title=vari   
        if 'units' in ds_f.attrs:
            units[i-1]=ds_f.units        
        year=str(date_ini.month)+'-'+str(date_ini.year)
        #unit=
        ax=plt.subplot(xp,yp,i,projection=proj)    
        ax=plt_xr_map_ax(ax,fig,data,lat,lon,land_50m,title,year,ranges[i-1],units[i-1],function=function,color_map=cmap,msize=msize)
    #plt.tight_layout()

    plt.subplots_adjust(wspace=0.07, hspace=0.03)
    
    if save_dir=='':
        save_dir = os.path.dirname(os.path.realpath('__file__'))
        save_name=var[0]
    
    if save:
        plt.savefig(save_dir+'/'+save_name)
        

    
    
    
    
    


from setuptools import setup, find_packages


packages = find_packages()

setup(
    name="sealeveltools",
    version='0.0.0',
    description='A project to handle sea level data (altimetry, tide-gauges, models) and statistical exploitation tools',
    license='',
    author='Julius Oelsmann',
    author_email='julius.oelsmann@tum.de',
    packages=packages,
    url="https://gitlab.lrz.de/iulius/sea_level_tool.git"
    #package_data={    },
    #entry_points={    },
    #setup_requires=["pytest-runner"],
)

